\ Open Memory Test Language parser
\
\ Copyright 2006 Dennis Ruffer (druffer@worldnet.att.net)
\
\     This program is free software; you can redistribute it and/or modify
\     it under the terms of the GNU General Public License as published by
\     the Free Software Foundation; either version 2 of the License, or
\     (at your option) any later version.
\
\     This program is distributed in the hope that it will be useful,
\     but WITHOUT ANY WARRANTY; without even the implied warranty of
\     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
\     GNU General Public License for more details.
\
\     You should have received a copy of the GNU General Public License
\     along with this program; if not, write to the Free Software
\     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
\
\ Concepts translated from the a lex & yacc front-end to:
\
\ Ad J. van de Goor, Aad Offerman, and Ivo Schanstra:
\ Towards a Uniform Notation for Memory Tests
\ presented at the European Design & Test Conference and Exhibition 96
\
\ In here you can find a predecessor to the language above that is much more
\ limited (more like an extension to the March test notation) and thus
\ much more cleaner.
\
\ Memory Test Verification and Generation
\  Copyright 1995, A. Offerman (offerman@einstein.et.tudelft.nl),
\                  H.I. Schanstra (ivo@duteca.et.tudelft.nl),
\                  Section Computer Architecture & Digital Systems,
\                  Department of Electrical Engeneering,
\                  Delft University of Technology, The Netherlands
\  All rights reserved
\
\ http://www.offerman.net/memory/mtl.tgz

\ Operations and outputs

\ There are two levels of operations;
\ operations on an entire memory chip and
\ operations on the memory cell array part of a memory

require ../ffl/stc.fs
require ../Structs.fth
require State_mask.fth

: Core_operation:   char: ;				\ Represents the type of operation

  0 constant read_operation
  1 constant write_operation
  2 constant no_operation
  3 constant dont_care_operation
  4 constant nest_operation

struct: Array_operation					\ An operation on the memory cell array
    Core_operation: array_core_operation
        State_mask: array_address		\ length n, 1 = cell accessed
        State_mask: array_data			\ length n = State_Digit(s)
              Flag: array_valid
;struct 

: .core_operation ( n -- )   case
		read_operation		of  ." read_operation "			endof
		write_operation		of  ." write_operation "		endof
		no_operation		of  ." no_operation "			endof
		dont_care_operation of  ." dont_care_operation "	endof
		nest_operation      of  ." nest_operation"          endof
	endcase
;

\ A vector of operations is used for multiple operations
\ simultaneously using different ports of a memory

: Array_operation_vector:   cell: ;		\ Vectors per port containing Array_operation(s)

\ array_output: a mask of length n defining a return value for every cell in the
\ memory cell array for a certain port.  Contains State_digit(s).

\ array_output_vector: this is a vector of array_output(s), one for every port
\ of the memory relevant to the fault.
