\ Open Memory Test Language parser
\
\ Copyright 2006 Dennis Ruffer (druffer@worldnet.att.net)
\
\     This program is free software; you can redistribute it and/or modify
\     it under the terms of the GNU General Public License as published by
\     the Free Software Foundation; either version 2 of the License, or
\     (at your option) any later version.
\
\     This program is distributed in the hope that it will be useful,
\     but WITHOUT ANY WARRANTY; without even the implied warranty of
\     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
\     GNU General Public License for more details.
\
\     You should have received a copy of the GNU General Public License
\     along with this program; if not, write to the Free Software
\     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
\
\ Concepts translated from the a lex & yacc front-end to:
\
\ Ad J. van de Goor, Aad Offerman, and Ivo Schanstra:
\ Towards a Uniform Notation for Memory Tests
\ presented at the European Design & Test Conference and Exhibition 96
\
\ In here you can find a predecessor to the language above that is much more
\ limited (more like an extension to the March test notation) and thus
\ much more cleaner.
\
\ Memory Test Verification and Generation
\  Copyright 1995, A. Offerman (offerman@einstein.et.tudelft.nl),
\                  H.I. Schanstra (ivo@duteca.et.tudelft.nl),
\                  Section Computer Architecture & Digital Systems,
\                  Department of Electrical Engeneering,
\                  Delft University of Technology, The Netherlands
\  All rights reserved
\
\ http://www.offerman.net/memory/mtl.tgz

\ Several fault structures can be distinguished;
\ the logical fault is modeled using the fault and faultset datastructures,
\ while the topological fault is modeled in the memory chip datastructure.

require ../ffl/stc.fs
require ../Vectors.fth
require ../Structs.fth
require Forced_state.fth
require New_state.fth
require Effect_mask_list.fth
require Port.fth
require Fault_set.fth

struct: Fault
                    char_*: fault_name
                    char_*: fault_notation
    Forced_state_mask_list: fault_forced_state_mask_list
       New_state_mask_list: fault_new_state_mask_list
          Effect_mask_list: fault_effect_mask_list
         Neighborhood_list: fault_neighborhood_list
    Dim2_mirror_constraint field: fault_array_mirror_constraint
               Port_vector: fault_port_vector
                       int: fault_cell_size
;struct

: new_Fault ( _name -- )
    Fault_set @ if
        Fault_set dup vsize 0 vinsert
    else
        Fault 1 vnew Fault_set !
    then
    >in @  Fault_set vlast dup Fault_set# ! constant
    >in !  bl word count  here current_fault_set dup >r fault_name ! $c,
    here r@ fault_notation !
    not_allowed r@ fault_array_mirror_constraint horizontal c!
    not_allowed r@ fault_array_mirror_constraint vertical c!
    not_allowed r@ fault_array_mirror_constraint diagonal c!
    Neighborhood 2 vnew r@ fault_neighborhood_list !
    2 0 do
        1 current_fault_set fault_neighborhood_list i [v] height !
        1 current_fault_set fault_neighborhood_list i [v] width !
    loop
    Port_struct 4 vnew r@ fault_port_vector !
    4 0 do
        read_port       current_fault_set fault_port_vector i [v] ports_type !
        write_port      current_fault_set fault_port_vector i [v] ports_type !
        read_write_port current_fault_set fault_port_vector i [v] ports_type !
        dont_care_port  current_fault_set fault_port_vector i [v] ports_type !
    loop
    0 r> fault_cell_size !
;

: append_fault_neighbor ( n offset -- )
    current_fault_set fault_port_vector rot [v] swap + a_neighborhood_list
    current_fault_set fault_neighborhood_list
    append_neighborhood_list
;

: d_mirror_mca ( -- )
	cr ." d_mirror_mca " cr
;

: h_mirror_nbh ( nbh_number -- )
	cr ." h_mirror_nbh @ " . cr
;

: v_mirror_nbh ( nbh_number -- )
	cr ." v_mirror_nbh @ " . cr
;

: d_mirror_nbh ( nbh_number -- )
	cr ." d_mirror_nbh @ " . cr
;
