\ Open Memory Test Language parser
\
\ Copyright 2006 Dennis Ruffer (druffer@worldnet.att.net)
\
\     This program is free software; you can redistribute it and/or modify
\     it under the terms of the GNU General Public License as published by
\     the Free Software Foundation; either version 2 of the License, or
\     (at your option) any later version.
\
\     This program is distributed in the hope that it will be useful,
\     but WITHOUT ANY WARRANTY; without even the implied warranty of
\     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
\     GNU General Public License for more details.
\
\     You should have received a copy of the GNU General Public License
\     along with this program; if not, write to the Free Software
\     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
\
\ Concepts translated from the a lex & yacc front-end to:
\
\ Ad J. van de Goor, Aad Offerman, and Ivo Schanstra:
\ Towards a Uniform Notation for Memory Tests
\ presented at the European Design & Test Conference and Exhibition 96
\
\ In here you can find a predecessor to the language above that is much more
\ limited (more like an extension to the March test notation) and thus
\ much more cleaner.
\
\ Memory Test Verification and Generation
\  Copyright 1995, A. Offerman (offerman@einstein.et.tudelft.nl),
\                  H.I. Schanstra (ivo@duteca.et.tudelft.nl),
\                  Section Computer Architecture & Digital Systems,
\                  Department of Electrical Engeneering,
\                  Delft University of Technology, The Netherlands
\  All rights reserved
\
\ http://www.offerman.net/memory/mtl.tgz

: mtl_compute_attributes ( -- exit )   0 mtl_variant ! \ mtla
    1 mtla_horizontal_period !
    1 mtla_vertical_period !
    1 mtla_bits_per_word !

    begin  mtla_port_type_vector vsize while
        mtla_port_type_vector 0 vextract
    repeat

    algorithm @+ 2dup aparse 0 >r
    over + mtl_test_sentinel !
    dup mtl_test_current !
    mtl_test ! r>

    mtla_horizontal_period @ 1- current_test horizontal_margin !
    mtla_vertical_period @ 1- current_test vertical_margin !
    mtla_bits_per_word @ current_test word_width !
;

: mtl_preprocessor ( -- exit )   2 mtl_variant ! \ mtlp
    begin  mtlp_port_type_vector vsize while
        mtlp_port_type_vector 0 vextract
    repeat

    0 mtlp_all_ports !
    0 mtlp_current_port !

    algorithm @+ 2dup pparse 0 >r
    over + mtl_test_sentinel !
    dup mtl_test_current !
    mtl_test ! r>

    mtlp_port_type_vector vsize 0= if
        mtla_port_type_vector @ current_test test_port_type_vector !
    else
        mtla_port_type_vector vsize
        mtlp_port_type_vector vsize = if
            mtlp_port_type_vector vsize 0 ?do
                mtlp_port_type_vector i [v] c@ read_port =
                mtla_port_type_vector i [v] c@ write_port =
                mtla_port_type_vector i [v] c@ read_write_port = or and
                mtlp_port_type_vector i [v] c@ write_port =
                mtla_port_type_vector i [v] c@ read_port =
                mtla_port_type_vector i [v] c@ read_write_port = or and or
                abort" MTL preprocessor error: port specification does not match test algorithm 1"
                mtlp_port_type_vector i [v] c@ dont_care_port = if
                    mtla_port_type_vector i [v] c@
                    mtlp_port_type_vector i [v] c!
                then
            loop
            mtlp_port_type_vector @ current_test test_port_type_vector !
        else
            true abort" MTL preprocessor error: port specification does not match test algorithm 2"
        then
    then
    mtlp_all_ports @ current_test test_port_type_vector vlast and
    abort" MTL preprocessor error: port specification does not match test algorithm 3"
    mtlp_test @ mtlp_reduced_test !
    begin  mtlp_reduced_test @ c@ [char] { <>
        mtlp_reduced_test @ mtlp_test_sentinel @ <> and while
            1 mtlp_reduced_test +!
    repeat
;

variable tmp_mtle_test_pointer
variable tmp_mtlp_test_sentinel

: $Sub ( replace.len compare.len -- flag )
	tmp_mtle_test_pointer @ over compare 0= dup if
		-rot dup >r tmp_mtle_test_pointer @ swap move
		r> 1- tmp_mtle_test_pointer +!
	else  nip nip  then
;

: cSub ( replace compare -- flag )
	tmp_mtle_test_pointer @ c@ = dup if
		swap tmp_mtle_test_pointer @ c!
	else  nip  then
;

: updown->down ( -- )
	s" DOWN  " s" UPDOWN" $Sub
	s" down  " s" updown" $Sub or
	  "down"     "updown" cSub or if
	    1 mtle_updown_dirty_tag !
	then
;

: updown->up ( -- )
	s" UP    " s" UPDOWN" $Sub
	s" up    " s" updown" $Sub or
	  "up"       "updown" cSub or if
	    1 mtle_updown_dirty_tag !
	then
;

: northsouth->south ( -- )
	s" SOUTH     " s" NORTHSOUTH" $Sub
	s" south     " s" northsouth" $Sub or
	  "south"        "northsouth" cSub or if
	    1 mtle_northsouth_dirty_tag !
	then
;

: northsouth->north ( -- )
	s" NORTH     " s" NORTHSOUTH" $Sub
	s" north     " s" northsouth" $Sub or
	  "north"        "northsouth" cSub or if
	    1 mtle_northsouth_dirty_tag !
	then
;

: eastwest->east ( -- )
	s" EAST    " s" EASTWEST" $Sub
	s" east    " s" eastwest" $Sub or
	  "east"       "eastwest" cSub or if
		1 mtle_eastwest_dirty_tag !
	then
;

: eastwest->west ( -- )
	s" WEST    " s" EASTWEST" $Sub
	s" west    " s" eastwest" $Sub or
	  "west"       "eastwest" cSub or if
		1 mtle_eastwest_dirty_tag !
	then
;

: setup_compare ( vector -- )   dup vdelete
    1 mtlp_test_sentinel @ mtle_test @ - dup >r vnew dup rot !
    mtle_test @ over r@ move
    dup tmp_mtle_test_pointer !
    r> + tmp_mtlp_test_sentinel !
;

: mtl_expand_test ( -- )   2 mtl_variant ! \ mtlp
    mtlp_reduced_test @ mtle_test !
    0 mtle_updown_dirty_tag !
    0 mtle_northsouth_dirty_tag !
    0 mtle_eastwest_dirty_tag !

    mtle_down_south_east_test setup_compare
    begin  tmp_mtle_test_pointer @ tmp_mtlp_test_sentinel @ < while
        updown->down  northsouth->south  eastwest->east
        1 tmp_mtle_test_pointer +!
    repeat

    mtle_up_south_east_test setup_compare
    begin  tmp_mtle_test_pointer @ tmp_mtlp_test_sentinel @ < while
        updown->up  northsouth->south  eastwest->east
        1 tmp_mtle_test_pointer +!
    repeat

    mtle_down_north_east_test setup_compare
    begin  tmp_mtle_test_pointer @ tmp_mtlp_test_sentinel @ < while
        updown->down  northsouth->north  eastwest->east
        1 tmp_mtle_test_pointer +!
    repeat

    mtle_up_north_east_test setup_compare
    begin  tmp_mtle_test_pointer @ tmp_mtlp_test_sentinel @ < while
        updown->up  northsouth->north  eastwest->east
        1 tmp_mtle_test_pointer +!
    repeat

    mtle_down_south_west_test setup_compare
    begin  tmp_mtle_test_pointer @ tmp_mtlp_test_sentinel @ < while
        updown->down  northsouth->south  eastwest->west
        1 tmp_mtle_test_pointer +!
    repeat

    mtle_up_south_west_test setup_compare
    begin  tmp_mtle_test_pointer @ tmp_mtlp_test_sentinel @ < while
        updown->up  northsouth->south  eastwest->west
        1 tmp_mtle_test_pointer +!
    repeat

    mtle_down_north_west_test setup_compare
    begin  tmp_mtle_test_pointer @ tmp_mtlp_test_sentinel @ < while
        updown->down  northsouth->north  eastwest->west
        1 tmp_mtle_test_pointer +!
    repeat

    mtle_up_north_west_test setup_compare
    begin  tmp_mtle_test_pointer @ tmp_mtlp_test_sentinel @ < while
        updown->up  northsouth->north  eastwest->west
        1 tmp_mtle_test_pointer +!
    repeat
;

variable chip_counter

: find_fault ( vector -- flag )   chip.reset
    ( vector ) vstring ['] iparse catch dup if
    	." Failure found " nip nip  then
    mtli_fault_found @ 0= or
    mtl_error @ or
;

: mtl_verify_chip ( chip -- complete )   1 mtl_variant ! \ mtli
    true mtli_fault_found !
    ( chip ) Memory_chip# !

    mtle_down_south_east_test find_fault if  0 exit  then

    mtle_updown_dirty_tag @ if
        mtle_up_south_east_test find_fault if  0 exit  then
    then

    mtle_northsouth_dirty_tag @ if
        mtle_down_north_east_test find_fault if  0 exit  then

		mtle_updown_dirty_tag @ if
            mtle_up_north_east_test find_fault if  0 exit  then
		then
    then

    mtle_eastwest_dirty_tag @ if
        mtle_down_south_west_test find_fault if  0 exit  then

		mtle_updown_dirty_tag @ if
            mtle_up_south_west_test find_fault if  0 exit  then
		then

		mtle_northsouth_dirty_tag @ if
            mtle_down_north_west_test find_fault if  0 exit  then

			mtle_updown_dirty_tag @ if
                mtle_up_north_west_test find_fault if  0 exit  then
			then
		then
    then
    chip_counter 1 over +! ?
    1   \ return ok
;

' mtl_verify_chip is verify_chip

: xmtvg_verify_for_completeness ( -- )
    0 chip_counter !
    0 mtl_error !
    cr Test_set# ? ." Name: " current_test test_name @ count type cr
    mtl_compute_attributes if ." mtl_compute_attributes failed"
    else  ." Horizontal period: " current_test horizontal_margin @ . cr
        ." Vertical period: " current_test vertical_margin @ . cr
        ." Bits per word: " current_test word_width @ . cr
        mtl_preprocessor if  ." mtl_preprocessor failed"
	else  ." Number of ports: " current_test test_port_type_vector vsize . cr
            mtl_expand_test
	." Number of memory chips processed: "
            mtl_error @ 0= if
                verify_completeness if
                    cr ." Complete"
                else  cr ." Incomplete"
                then
            else  cr ." Error"
            then
        then
    then  cr
;
