\ MTL Parser based on gforth's Gray version 5
\
\ Copyright 2006 Dennis Ruffer (druffer@worldnet.att.net)
\
\     This program is free software; you can redistribute it and/or modify
\     it under the terms of the GNU General Public License as published by
\     the Free Software Foundation; either version 2 of the License, or
\     (at your option) any later version.
\
\     This program is distributed in the hope that it will be useful,
\     but WITHOUT ANY WARRANTY; without even the implied warranty of
\     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
\     GNU General Public License for more details.
\
\     You should have received a copy of the GNU General Public License
\     along with this program; if not, write to the Free Software
\     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
\

require gray.fs

s" max-char" environment? 0= [IF]  255  [THEN]
1+ dup constant eof-char   max-member   \ the whole character set

2variable input   \ a string in 'addr count' representation
2variable ident   \ a string in 'addr count' representation

create tokens   eof-char 1+ cells allot

variable tokenval   0 tokenval !

: token ( -- ) ( use: token name ) ( name: -- n )
	( defines a token that returns a unique value )
	tokenval @ constant
	last @ tokenval @ cells tokens + !		\ save name pointers
	1 tokenval +! ;

: .token ( n -- )
	dup [char] a [char] z 1+ within if
		emit space
	else
		?dup if  cells tokens + @ id.
		else
			ident 2@ type space
		then
	then
;

token 'unsigned_integer'
token 'string'
token 'eof'

token '['	token ']'
token '('	token ')'
token '{'	token '}'
token ':'	token ';'
token '+'	token '-'
token '*'	token '='
token ','

wordlist constant keyword-wordlist
get-current ( * ) keyword-wordlist set-current

token ALLPORTS
token DOWN		token UP		token UPDOWN
token SOUTH		token NORTH		token NORTHSOUTH
token EAST		token WEST		token EASTWEST
token NOP

( * ) set-current

: adds ( addr1 c -- addr1+1 )   \ accumulates char to string
	over c! 1+
;

: key/ident ( addr u -- n )		\ checks string keyword and returns token value
	2dup keyword-wordlist search-wordlist if  nip nip execute
	else  drop c@ 32 or			\ return lcc if keyword not found
	then ;

30 stack expected

: ch ( -- char )
	input 2@ if  c@
	else  drop eof-char
	then
;

: testchar? ( set -- f )   dup expected push  ch member? ;
' testchar? test-vector !

: ?syntax-error ( f -- )
	0= if  empty
		begin
			expected top union
			expected pop
			expected clear?
		until  ." expected: "
		['] emit apply-to-members cr
		true abort" syntax error"
	then
;

: ?readnext ( f -- )
	?syntax-error
	expected clear
	input 2@ dup if
		1 /string
	then  input 2!
;

: init ( str len -- )   input 2! ;

: .. ( c1 c2 -- set )   \ creates set including the characters c, c1<=c<=c2
	empty copy-set
	swap 1+ rot do
		i over add-member
	loop
;

: ` ( -- terminal ) ( use: ` c )   \ creates anonymous terminal for character c
	char singleton ['] ?readnext make-terminal
;

char a char z ..  char A char Z ..  union	' ?readnext terminal letter
char 0 char 9 ..							' ?readnext terminal digit
0 32 ..										' ?readnext terminal blanks
eof-char singleton							' ?readnext terminal eof-scanner

(( blanks **
	(( ` [			{{ '[' }}
	|| ` ]			{{ ']' }}
	|| ` (			{{ '(' }}
	|| ` )			{{ ')' }}
	|| ` {			{{ '{' }}
	|| ` }			{{ '}' }}
	|| ` ,			{{ ',' }}
	|| ` :			{{ ':' }}
	|| ` ;			{{ ';' }}
	|| ` =			{{ '=' }}
	|| ` +			{{ '+' }}
	|| ` -			{{ '-' }}
	|| ` *			{{ '*' }}
	|| eof-scanner	{{ 'eof' }}
	|| {{ pad 0 ident 2!  pad ch adds }} digit
		(( {{ ch adds }} digit )) **
		{{ pad - pad swap ident 2!  'unsigned_integer' }}
	|| {{ pad 0 ident 2!  pad ch adds }} letter
		(( {{ ch adds }} letter )) **
		{{ pad - pad swap 2dup ident 2!  key/ident }}
	))
)) <- symbol

symbol parser scanner

: tscan ( str len -- )   init
	begin  scanner \ dup .token
		'eof' =
	until
;

\ parser
tokenval @ 1- char a > [if]  ." too many tokens "  abort  [then]
char z 1+ max-member

variable sym

: testsym? ( set -- f )   sym @ member? ;
' testsym? test-vector !

: ?nextsym ( f -- )
	0= abort" syntax error "
	scanner \ dup .token
	sym !
;

: t ( n -- ) ( use: token t name )   singleton ['] ?nextsym terminal ;

get-order keyword-wordlist swap 1+ set-order

'unsigned_integer' t <unsigned_integer>
'string' t <string>
'eof' t <eof>

'[' t <[>	']' t <]>
'(' t <(>	')' t <)>
'{' t <{>	'}' t <}>
':' t <:>	';' t <;>
'+' t <+>	'-' t <->
'*' t <*>	'=' t <=>
',' t <,>

ALLPORTS t <ALLPORTS>
    DOWN t <DOWN>		   UP t <UP>		    UPDOWN t <UPDOWN>
   SOUTH t <SOUTH>		NORTH t <NORTH>		NORTHSOUTH t <NORTHSOUTH>
    EAST t <EAST>		 WEST t <WEST>		  EASTWEST t <EASTWEST>
     NOP t <NOP>

char a t <a>	char b t <b>	char c t <c>	char d t <d>	char e t <e>
char f t <f>	char g t <g>	char h t <h>	char i t <i>	char j t <j>
char k t <k>	char l t <l>	char m t <m>	char n t <n>	char o t <o>
char p t <p>	char q t <q>	char r t <r>	char s t <s>	char t t <t>
char u t <u>	char v t <v>	char w t <w>	char x t <x>	char y t <y>
char z t <z>

previous
